"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const isPromise = (obj) => typeof obj !== 'undefined' && typeof obj.then === 'function';
/** Functions for when a result may be either synchronous (a value) or asynchronous (a `Promise`) */
var PromiseOrValue;
(function (PromiseOrValue) {
    /** Apply a transformation to the result of a promise or value */
    function map(promiseOrValue, onFulfilled) {
        if (isPromise(promiseOrValue)) {
            return promiseOrValue.then(onFulfilled);
        }
        else {
            return onFulfilled(promiseOrValue);
        }
    }
    PromiseOrValue.map = map;
    /** Attach callbacks for the resolution/rejection of the result of the action */
    function then(action, onFulfilled, onRejected) {
        try {
            const result = action();
            if (isPromise(result)) {
                return result.then(onFulfilled, onRejected);
            }
            else {
                return onFulfilled(result);
            }
        }
        catch (err) {
            if (typeof onRejected !== 'undefined') {
                return onRejected(err);
            }
            else {
                throw err;
            }
        }
    }
    PromiseOrValue.then = then;
    /** Handle errors thrown by the action */
    function catchErr(action, onRejected) {
        return then(action, x => x, onRejected);
    }
    PromiseOrValue.catchErr = catchErr;
    /** Turn a promise or value into a promise */
    function asPromise(promiseOrValue) {
        if (isPromise(promiseOrValue)) {
            return promiseOrValue;
        }
        else {
            return Promise.resolve(promiseOrValue);
        }
    }
    PromiseOrValue.asPromise = asPromise;
})(PromiseOrValue = exports.PromiseOrValue || (exports.PromiseOrValue = {}));
const makeSpeech = (speech) => {
    if (typeof speech.SSML !== 'undefined') {
        return {
            type: 'SSML',
            ssml: speech.SSML,
        };
    }
    else if (typeof speech.Text !== 'undefined') {
        return {
            type: 'PlainText',
            text: speech.Text,
        };
    }
    else {
        return undefined;
    }
};
const makeCard = (card) => {
    if (card.Type === 'LinkAccount') {
        return {
            type: 'LinkAccount',
        };
    }
    if (card.Image !== undefined) {
        return {
            type: 'Standard',
            title: card.Title,
            text: card.Content,
            image: {
                largeImageUrl: card.Image.LargeUrl,
                smallImageUrl: card.Image.SmallUrl,
            },
        };
    }
    return {
        type: 'Simple',
        title: card.Title,
        content: card.Content,
    };
};
const makeResponse = (response) => {
    const output = {
        shouldEndSession: response.EndSession || false,
    };
    if (response.Say !== undefined) {
        const speech = makeSpeech(response.Say);
        if (typeof speech !== 'undefined') {
            output.outputSpeech = speech;
        }
    }
    if (response.Reprompt !== undefined) {
        const speech = makeSpeech(response.Reprompt);
        if (typeof speech !== 'undefined') {
            output.reprompt = { outputSpeech: speech };
        }
    }
    if (response.Card !== undefined) {
        output.card = makeCard(response.Card);
    }
    return output;
};
/** Functions for accessing raw AlexaTs state */
var State;
(function (State) {
    /** The string key used to store state from AlexaTs in the Alexa session attributes */
    State.attributeKey = '_alexaTsState';
    /** Get the state (if available) from the request, otherwise return the initial state. */
    function fromRequest(request, initialState) {
        if (request &&
            request.session &&
            request.session.attributes &&
            State.attributeKey in request.session.attributes) {
            return request.session.attributes[State.attributeKey];
        }
        else {
            return initialState;
        }
    }
    State.fromRequest = fromRequest;
})(State = exports.State || (exports.State = {}));
const sessionAttributesFromResponse = (response, previousState) => {
    let sessionAttributes = {};
    if ('NewState' in response) {
        sessionAttributes[State.attributeKey] = response.NewState;
    }
    else if (typeof previousState !== 'undefined') {
        sessionAttributes[State.attributeKey] = previousState;
    }
    return sessionAttributes;
};
/** Expand a short-hand `Response` into a raw Alexa `ResponseBody` */
exports.response = (response, previousState) => {
    return {
        version: '1.0',
        response: makeResponse(response),
        sessionAttributes: sessionAttributesFromResponse(response, previousState),
    };
};
const handlerObjToMap = (obj) => {
    const map = new Map();
    Object.keys(obj).forEach(key => map.set(`AMAZON.${key}Intent`, obj[key]));
    return map;
};
const slotsToMap = (slots) => {
    const map = new Map();
    if (slots instanceof Object) {
        Object.keys(slots).forEach(key => map.set(slots[key].name, slots[key].value));
    }
    return map;
};
const buildRequestIfNeeded = (outputOrResponse, previousState) => {
    if (typeof outputOrResponse !== 'object') {
        return;
    }
    if (outputOrResponse['Say'] !== undefined) {
        return exports.response(outputOrResponse, previousState);
    }
    else {
        return outputOrResponse;
    }
};
const mapIntentResult = (result, state) => PromiseOrValue.map(result, output => buildRequestIfNeeded(output, state));
/** Functions for creating and composing middleware `Pipe`s */
var Pipe;
(function (Pipe) {
    /** Create a pipe by calling a series of child pipes. */
    Pipe.join = (steps) => (event, next) => {
        const processNext = (remainingHandlers) => (event) => {
            if (remainingHandlers.length === 0) {
                if (typeof next !== 'undefined') {
                    return next(event);
                }
                else {
                    throw new Error('Event unhandled');
                }
            }
            else {
                const nextHandler = remainingHandlers[0];
                const newRemaining = remainingHandlers.slice(1);
                return nextHandler(event, processNext(newRemaining));
            }
        };
        return processNext(steps)(event);
    };
    /** Convert a pipe into a handler. */
    Pipe.toHandler = (pipe) => event => pipe(event, () => {
        throw new Error('Event unhandled');
    });
    /** Handle specific request types and intents. */
    Pipe.router = (routes) => {
        const standardIntents = handlerObjToMap(routes.Standard || {});
        const customIntents = new Map(routes.Custom || []);
        return (event, next) => {
            const state = State.fromRequest(event, routes.InitialState);
            switch (event.request.type) {
                case 'LaunchRequest':
                    if (routes.Launch !== undefined) {
                        return mapIntentResult(routes.Launch(state, new Map(), event, next), state);
                    }
                    break;
                case 'SessionEndedRequest':// Special case - can't respond.
                    if (routes.SessionEnded !== undefined) {
                        return routes.SessionEnded();
                    }
                    return;
                case 'IntentRequest':
                    const intentRequest = event.request;
                    const slots = slotsToMap(intentRequest.intent.slots);
                    const standardHandler = standardIntents.get(intentRequest.intent.name);
                    if (standardHandler !== undefined) {
                        return mapIntentResult(standardHandler(state, slots, event, next), state);
                    }
                    const customHandler = customIntents.get(intentRequest.intent.name);
                    if (customHandler !== undefined) {
                        return mapIntentResult(customHandler(state, slots, event, next), state);
                    }
            }
            return next(event);
        };
    };
    /** Catch and handle errors from subsequent handlers. */
    Pipe.catchErr = (onError) => (request, next) => PromiseOrValue.catchErr(() => next(request), onError);
    /** Log the incoming request and the response or error thrown by the subsequent handlers. */
    Pipe.tracer = (logger) => {
        const log = logger !== undefined
            ? logger
            : (message, obj) => console.log(message, JSON.stringify(obj));
        return (event, next) => {
            log('Request:', event);
            try {
                const result = next(event);
                if (isPromise(result)) {
                    return result.then(response => {
                        log('Response:', response);
                        return response;
                    }, error => {
                        log('Error:', error.toString());
                        throw error;
                    });
                }
                else {
                    log('Response:', result);
                    return result;
                }
            }
            catch (error) {
                log('Error:', error.toString());
                throw error;
            }
        };
    };
    /** Just call the next step in the pipe. */
    Pipe.doNothing = () => (event, next) => next(event);
})(Pipe = exports.Pipe || (exports.Pipe = {}));
exports.Handler = {
    middleware: Pipe.toHandler,
    router: (routes) => Pipe.toHandler(Pipe.router(routes)),
    pipe: (steps) => Pipe.toHandler(Pipe.join(steps)),
};
const lambdaFromHandler = (handler) => (event, context, callback) => {
    PromiseOrValue.then(() => handler(event), data => {
        if (event &&
            event.request &&
            event.request.type === 'SessionEndedRequest') {
            callback(null);
        }
        else {
            callback(null, data);
        }
    }, callback);
};
/** Functions for creating an AWS lambda handler */
var Lambda;
(function (Lambda) {
    /** Create a lambda handler from an Alexa Handler */
    Lambda.handler = lambdaFromHandler;
    /** Create a lambda handler from request & intent routes */
    Lambda.router = (routes) => lambdaFromHandler(exports.Handler.router(routes));
    /** Create a lambda handler from a middleware pipeline */
    Lambda.pipe = (steps) => lambdaFromHandler(exports.Handler.pipe(steps));
})(Lambda = exports.Lambda || (exports.Lambda = {}));
//# sourceMappingURL=index.js.map